*-- File: INDEX.CC

CLASS Index
*----------------------------------------------------------------------
*-- Description:
*
*      This class contains methods designed to assist working with
*      indexes, specifically .MDX tags for the dBASE .DBF.
*
*-- Programmers: 
*
*      Ken Mayer
*      Rose Vines (CIS: 100026,3153)
*      Bowen Moursund (CIS: 72662,436)
*      David Love (CIS: 70153,2433)
*
*-- History: 
*
*      11/25/1996 -- original version of these routines as a class.
*
*-- Instantiation: 
*
*      set procedure to index.cc additive
*      _app.Index = new Index()
*
*-- Methods:
*      More details on each of these is contained in the commentary for
*      each method ...
*
*--    Init()
*        Initializes classes needed
*
*--    Release()
*        Releases this object from memory
*
*--    DelMDX()
*        Deletes mdx and fixes dbf header
*
*--    FindTagExp()
*        Finds starting position of specific index tag expression
*
*--    MDXByte()
*        Turn on/off MDX byte in .DBF header ...
*
*--    MDXPoint()
*        Changes the hard-coded DBF name in an MDX file header
*
*--    PrntTags()
*        Prints index tags, and optionally structure of table
*
*--    ReDoTags()
*        A "generic" MDX cleanup routine
*
*-- Dependencies:
*
*      CONVERT.CC
*      FILEEX.CC
*      DUFLP.H
*----------------------------------------------------------------------

   CLASS::Init()

   FUNCTION Init
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/10/1996
   *-- Notes.......: Ensures that routines needed are available
   *--               It is required that this be used (see instructions
   *--               above) for this class to function properly.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/10/1996 -- Original Code
   *-- Calls.......: FILEEX.CC, CONVERT.CC
   *-- Usage.......: index.Init()
   *-- Example.....: _app.Index.Init()
   *-- Returns.....: Nul
   *-- Parameters..: None
   *-------------------------------------------------------------------
   
      PROTECT ConvertProc, FileProc
      
      if .not. SetProc( "CONVERT.CC" )
         set procedure to convert.cc additive
         this.ConvertProc = .t.
      else
         this.ConvertProc = .f.
      endif
      this.Convert        = new Convert()
      this.Convertparent  = this
      if .not. SetProc( "FILEEX.CC" )
         set procedure to fileex.cc  additive
         this.FileProc = .t.
      else
         this.FileProc = .f.
      endif
      this.File           = new FileEx()
      this.File.parent    = this
      
   RETURN NULL

   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: index.Release()
   *-- Example.....: _app.Index.Release()
   *--               _app.Index = NULL  && not required, good idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------
      if this.ConvertProc
         close procedure convert.cc
      endif
      if this.FileProc
         close procedure fileex.cc
      endif
      release object this
   
   RETURN NULL
   *-- EoM: Release()

   FUNCTION DelMdx( cMdx )
   *-------------------------------------------------------------------
   *-- Programmer..: Rose Vines (CIS: 100026,3153)
   *-- Date........: 11/10/1996
   *-- Notes.......: Deletes a production index file, correctly 
   *--               updating the production index byte in the DBF 
   *--               header, so you avoid the "Production index not 
   *--               found" message.
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 04/26/1993  -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302                                     ,
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: index.DelMdx( <cMdx> )
   *-- Example.....: _app.Index.DelMdx("fred")
   *-- Returns.....: nError  =  0 if deletion is okay
   *--                          1 if file doesn't exist
   *-- Parameters..: cMdx = Production MDX file to delete
   *-------------------------------------------------------------------
      
      local cMdx, cMdxName, cDbfName, nHandle, nError, xJunk
      
      cMdxName = cMdx + ".MDX"
      cDbfName = cMdx + ".DBF"
      nError   = 0
      
      *-- Check if file exists
      if file(cMdxName)
         *-- Delete MDX file
         delete file (cMdxName)
          
         *-- Update MDX byte in DBF header, indicating there is no 
         *-- longer an MDX for this database.
         nHandle = fopen((cDbfName),"rw")
         xJunk   = fseek(nHandle,28,0)
         xJunk   = fwrite(nHandle,chr(0))
         xJunk   = fclose(nHandle)
      else
         nError = 1
      endif

   RETURN nError
   *-- EoM: DelMdx()

   FUNCTION FindTagExp( nHandle, cMdxTag )
   *-------------------------------------------------------------------
   *-- Programmer..: Rose Vines (CIS: 100026,3153) (dBase version)
   *-- Date........: 11/10/1996
   *-- Notes.......: Finds the starting position of a specific index 
   *--               TAG expression within an MDX header.
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 04/26/1993  -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.   
   *-- Calls.......: fileex.FLocate()
   *--               convert.BreakLong()
   *-- Usage.......: index.FindTagExp( <nHandle>, <cMdxTag> )
   *-- Example.....: _app.Index.FindTagExp( 5, "upper(LASTNAME)+;
   *--                                       upper(FIRSTNAME)")
   *-- Returns.....: nTagExp   -  Starting position of the TAG 
   *--                            expression within the MDX header 
   *--                            file.
   *-- Parameters..: nHandle  =  DBASE file handle of an MDX file.
   *--               cMdxTag  =  MDX TAG expression.
   *-------------------------------------------------------------------
      
      local nPos, nPoint, cLong, nTagExp
      
      *-- Shift pointer to byte 512 in the MDX file. At byte 512,
      *-- there's an array of TAG names.
      fseek( nHandle, 512, 0 )
      *-- From there, locate our particular TAG in the array
      nPos    = _app.Table.Locate( nHandle, cMdxTag, .T. )
      *-- Back up and read the preceding 4 bytes, which are a pointer
      *-- to the file offset where the information about our TAG
      *-- is located in the MDX file.
      fseek( nHandle, nPos - 4 )
      *-- Convert the 4-byte pointer to decimal
      cLong = fread( nHandle, 4 )
      nPoint  = this.Convert.BreakLong( cLong, 1 )
      *-- Return the starting position of the TAG info.
      nTagExp = fseek( nHandle, nPoint * 512 )
      
   RETURN ( nTagExp )
   *-- EoM: FindTagExp()

   FUNCTION MDXbyte
   *-------------------------------------------------------------------
   *-- Programmer..: Bowen Moursund (CIS: 72662,436)
   *-- Date........: 10/28/1996
   *-- Notes.......: Sets the MDX byte in a DBF header ON or OFF.
   *--               The DBF must not be open when the function is 
   *--               called.
   *-- Written for.: dBASE IV v1.5
   *-- Rev. History: 05/21/1992 -- Original
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: dBASE low level file functions
   *-- Usage.......: Index.MDXbyte(<cDBFpath>,<cOnOff>)
   *-- Example.....: lByteSet = _app.Index.MDXbyte("mydbf.dbf","OFF")
   *-- Returns.....: .T. if successful
   *-- Parameters..: cDBFpath = the [path]filename.ext of the DBF
   *--               cOnOff   = "ON" or "OFF"
   *-------------------------------------------------------------------

      parameters cDBFpath,cOnOff
      private all

      cOnOff = upper(cOnOff)
      * check the validity of the parameters
      lSuccess = ( pcount() = 2 .AND. cOnOff $ "ON|OFF" .AND. ;
                   file(cDBFpath) )
      if lSuccess
         nHandle = fopen(cDBFpath,"RW")
         if nHandle > 0
            if fseek(nHandle, 28) = 28
               lSuccess = ( fwrite(nHandle, ;
                          iif(cOnOff="OFF",chr(0),chr(1))) = 1 )
            else
               lSuccess = .F.
            endif
            lClosed = fclose(nHandle)
         else
            lSuccess = .F.
         endif
      endif

   RETURN lSuccess
   *-- EoM: MDXbyte()

   FUNCTION MdxPoint( cDbfName, cPath, cMdx )
   *-------------------------------------------------------------------
   *-- Programmer..: Rose Vines
   *-- Date........: 11/10/1996
   *-- Notes.......: Changes the hard-coded DBF name in an MDX file 
   *--               header (either a production or non-production MDX)
   *-- Written for.: dBASE IV, 2.0
   *--               This version will NOT compile under dBASE IV.
   *-- Rev. History: 04/26/1993 -- Original Release
   *--               09/25/1994 -- Andrew Clayton, revised to deal with
   *--                             possibility of filespec containing 
   *--                             path
   *--               09/27/1994 -- Jay Parsons, changed padding, 
   *--                             pcount()
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: Index.MdxPoint( <cDbfName>, <cPath> [,<cMdx>] )
   *-- Example.....: _app.Index.MdxPoint("FRED","A:\")
   *--               _app.Index.MdxPoint("FRED","A:\","FULLNAME")
   *-- Returns.....: None
   *-- Parameters..: cDbfName = The name of the DBF to be hard-coded 
   *--                          into the MDX header.
   *--               cPath    = the drive and directory for the MDX 
   *--                          file, use "" to signify current 
   *--                          directory
   *--               cMdx     = The name of the MDX file, if it's a
   *--                          non-production MDX (omit this parameter
   *--                          completely if it's a production MDX).
   *-------------------------------------------------------------------
      
      local cDbf, nHandle, n
      
      cDbf = upper(cDbfName)
      
      *-- Check how many parameters have been passed: 2 means its a
      *-- production index, 3 is a non-production index
      cMdx = iif( pcount() < 3, cDbf, cMdx )
      nHandle = fopen((cPath + cMdx)+".MDX","rw")
      
      *-- Position file pointer to Byte 4, which is start of hard-coded
      *-- DBF name in MDX header
      fseek(nHandle,4,0)

      *-- Write the new DBF filename into the header
      fwrite(nHandle, cDbf + replicate( chr(0), 8 - len( cDbf ) ) )
      fclose(nHandle)
      
   RETURN ("")
   *-- EoM: MdxPoint()

   FUNCTION PrntTags( cDBFParm, lDispStru )
   *-------------------------------------------------------------------
   *-- Programmer..: David Love (CIS: 70153,2433)
   *-- Date........: 12/02/1996
   *-- Notes.......: This routine is a "quick and not-so-dirty" method 
   *--               of printing the tag and key expressions for a 
   *--               dbf's production mdx file. It obviates the need 
   *--               for DISP or LIST STAT TO PRINT (or DISP STAT with 
   *--               SHIFT+PrtScr). This code is modified from the 
   *--               procedure RedoTags, previously posted on the
   *--               BORBBS.
   *--               The proc will print the full key expression,
   *--               including FOR/DESCENDING/UNIQUE options, if 
   *--               present.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 01/31/1992 - original procedure for dBASE IV, 
   *--                            Ver 1.1
   *--               04/18/1992 - David Love - revised for version 1.5
   *--               03/24/1993 - Lee Hite - modified so that wild card
   *--                            specs may now be used to list multiple 
   *--                            .DBF's. also, added optional parameter
   *--                            to include file structure in output.
   *--               12/02/1996 -- Modified for Visual dBASE, calls
   *--                             ChoosePrinter() ...
   *-- Calls.......: fileex.ParsePath()
   *-- Usage.......: do PrntTags with "<cDBF>",[lDispStru]
   *-- Example.....: do PrntTags with "Referral"
   *--               do PrntTags with "*.dbf",.t.
   *-- Returns.....: None
   *-- Parameters..: cDBF      = Name of DATABASE file, may include
   *--                           wildcard specs(i.e., "REF*").  No
   *--                           extension is necessary, but if it's
   *--                           there, it better be ".DBF" <g>
   *--               lDispStru = [optional] set to .T. to include the 
   *--                           file structure in the output
   *-------------------------------------------------------------------

      private cTalk
      cTalk = set("TALK")
      set talk off
      *-- default it if needed:
      if pCount() = 0
         cDBFParm = "*.DBF"
      endif

      if ChoosePrinter()
      
         *-- handle whether or not we got a wild card
         private cDBFPath,cDBFMask,nDBFs,lDummy,nKntr
         if "*" $ cDBFParm .or. "?" $ cDBFParm
            *-- wildcards, so build an array of the file names
            cDBFMask = iif(at(".DBF",upper(cDBFParm))>0,;
                           cDBFParm, cDBFParm+".DBF")
            aDirect = new Array()
            aDirect.dir( cDBFMask )
            nDBFs = aDirect.size/5
            if nDBFs > 0
               aMyArray = new Array( )
               nKntr = 1
               do while nKntr <= nDBFs
                  aMyArray.add( aDirect[nKntr,1] )
                  nKntr = nKntr + 1
               enddo
               aMyArray.sort()
            endif
            cDBFPath = this.File.ParsePath(cDBFMask)
         else
            *-- no wild cards, so we just have one entry in the array
            aMyArray = new Array()
            aMyArray.add( upper(cDBFParm) )
            nDBFs = 1
            cDBFPath = ""
         endif

         *-- Turn the durn printer on
         set printer on
         
         *-- loop for each .DBF
         private cDBF
         nKntr = 1
         do while nKntr <= nDBFs

            cDBF = aMyArray[nKntr]
            *-- pull extension out of file name so TAGCOUNT(), etc. 
            *-- work...
            cDBF = iif(at(".DBF",cDBF)=0,cDBF,;
                   left(cDBF,at(".DBF",cDBF)-1))
            use (cDBFPath+cDBF)
            ?? "TABLE: "+cDBF at 0
            ?
            ?
            
            *-- display file structure if optioned
            if lDispStru
               ?? "STRUCTURE:" at 0
               disp stru
               ?
            endif
    
            *-- now, figure out how many tags exist
            private nMaxTags
            nMaxTags = tagcount( cDBF )
            ?? "INDEX TAGS:" at 0
            ?
            
            *-- only perform routine if an index tag exists
            if nMaxTags > 0
               private nTags, mkey, mtag
               
               *-- store the keys and tags to an array
               aTags = new Array( nMaxTags, 5 )
               nTags = 1
               do while nTags <= nMaxTags
                  * grab the key
                  store key( (cDBF),nTags)        to aTags[nTags,1]
                  * grab the tagname
                  store tag( (cDBF),nTags)        to aTags[nTags,2]
                  * grab the for clause
                  store for( (cDBF),nTags)        to aTags[nTags,3]
                  * .t. if descending
                  store descending( (cDBF),nTags) to aTags[nTags,4]
                  * .t. if unique
                  store unique( (cDBF),nTags)     to aTags[nTags,5]
                  nTags = nTags + 1
               enddo
               
               *-- print each tag with it's key expression
               ?? "Tag" at 0
               ?? "Key Expression" AT 12
               ?
               nTags = 1
               do while nTags <= nMaxTags
                  ?? aTags[nTags,2] AT 0
                  ?? aTags[nTags,1] + ;
                     iif(""#aTags[nTags,3]," FOR "+;
                            aTags[nTags,3],"") +;
                     iif(aTags[nTags,4]," DESCENDING","") + ;
                     iif(aTags[nTags,5]," UNIQUE","") AT 12
                  ?
                  nTags = nTags + 1
               enddo

               *-- release the array ...
            else
               *-- no tags found
               ?? "none" at 0
               ?
            endif  && check for tags ...
            use    && close database
            ?? replicate("=",60) at 0
            ?

            nKntr = nKntr + 1
            
         enddo  && loop for each .dbf
      
         *-- turn off whatever printer was selected:
         set printer off
         close printer

      endif && chooseprinter()
      set talk &cTalk.
      
   RETURN NULL
   *-- EoM: PrntTags

   FUNCTION RedoTags(cDBF)
   *-------------------------------------------------------------------
   *-- Programmer..: David Love (CIS: 70153,2433)
   *-- Date........: 11/10/1996
   *-- Notes.......: This routine is a "generic" MDX cleanup routine. 
   *--               It is useful for handling "bloated" MDX files -- 
   *--               ones that have been around awhile (they tend to be 
   *--               larger than necessary). This routine will store 
   *--               the tag keys in an array, delete the tags, and 
   *--               then rebuild the MDX file from scratch, keeping 
   *--               all tag names and keys, and the MDX SHOULD be 
   *--               smaller. Will act on production mdx (ie. same name
   *--               as dbf)
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 01/20/1992 - original function for dBASE IV 
   *--                            Ver. 1.1.
   *--               04/18/1992 - David Love - adapted for dBASE IV, 
   *--                            1.5.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: index.RedoTags(<cDBF>)
   *-- Example.....: _app.Index.RedoTags("Referral")
   *-- Returns.....: NULL
   *-- Parameters..: cDBF = Name of DATABASE file, no extension 
   *--                      necessary.
   *-------------------------------------------------------------------
      
      local nMaxTags, nTags, aTags, mkey, mtag
      
      use (cDBF) excl
      
      *-- First, figure out how many tags exist
      nMaxTags = tagcount( cDBF, 1 )
      
      *-- only perform routine if an index tag exists
      if nMaxTags > 0
         
         *-- store the keys and tags to an array
         aTags = new Array( nMaxTags,5 )
         nTags = 1
         do while nTags <= nMaxTags
            && grab key
            store key( (cDBF),nTags) to aTags[nTags,1]
            && grab tagname
            store tag( (cDBF),nTags) to aTags[nTags,2]
            && grab for clause
            store for( (cDBF),nTags) to aTags[nTags,3]
            && .t. =descending
            store descending( (cDBF),nTags) to aTags[nTags,4]
            && .t. =unique
            store unique( (cDBF),nTags) to aTags[nTags,5]
            nTags = nTags + 1
         enddo
         
         *-- now, delete the tags
         do while "" # tag( (cDBF),1)
            delete tag tag( (cDBF), 1)
         enddo
         
         *-- rebuild the MDX, tag by tag ...
         nTags = 1
         do while nTags <= nMaxTags
            mkey = aTags[nTags,1]+iif(""#aTags[nTags,3]," ;
                   for "+aTags[nTags,3],"") ;
                  + iif(aTags[nTags,4]," DESCENDING","") ;
                  + iif(aTags[nTags,5]," UNIQUE","")
            mtag = aTags[nTags,2]
            index on &mkey. tag &mtag.
            nTags = nTags + 1
         enddo

      endif  && check for tags ...
      use    && close database

   RETURN NULL
   *-- EoM: RedoTags()

ENDCLASS
*-- End of Class: INDEX

#INCLUDE <dUFLP.H>

*-----------------------------------------------------------------------
*-- End of File: INDEX.CC
*-----------------------------------------------------------------------