*-- File: TIME.CC

CLASS Time
*----------------------------------------------------------------------
*-- Description:
*
*      This class contains a set of record-processing routines.
*
*-- Programmers: 
*
*      Ken Mayer
*
*-- History: 
*
*      11/27/1996 -- original version of these routines as a class.
*
*-- Instantiation: 
*
*      set procedure to time.cc additive
*      _app.Time = new Time()
*
*-- Methods:
*      More details on each of these is contained in the commentary for
*      each method ...
*   
*--    Release()
*        Releases this object (and others) from memory
*
*--    AddTimes()
*        Adds two time strings together
*
*--    Civ2Mil()
*        Converts 12 hour to military 24 hour time.
*
*--    DiffTime()
*        Difference between two time strings
*
*--    IsAmPm()
*        Is time string in proper am/pm format.
*
*--    Mil2Civ()
*        Converts 24 hour to 12 hour time.
*
*--    Sec2Time()
*        Number of seconds converted to time string
*
*--    Time2Sec()
*        Converts time string to number of seconds
*
*-- Properties:
*
*      None
*----------------------------------------------------------------------

   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: time.Release()
   *-- Example.....: _app.Time.Release()
   *--               _app.Time = NULL   && not required, but a good 
   *--                                  && idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------

      release object this

   RETURN NULL
   *-- EoM: Release()

   FUNCTION AddTimes( cTime1, xTime2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Angus Scott-Fleming (CIS: 76711,517)
   *-- Date........: 11/09/1996
   *-- Notes.......: Add two times
   *-- Written for.: dBASE IV
   *-- Rev. History: 05/28/1992 -- Original
   *--               11/09/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: time.Time2Sec()   
   *--               time.Sec2Time()    
   *-- Usage.......: time.AddTimes(<cTime1>,<cTime2>)
   *--                OR 
   *--               time.AddTimes(<cTime1>,<nHours>)
   *-- Example.....: ? _app.Time.AddTimes("01:24:15","02:00:00") 
   *--                    OR 
   *--               ? _app.Time.AddTimes("01:24:15",2.5)
   *-- Returns.....: cTime    = Time string in format 
   *--                                    HH:MM:SS/HH:MM:SS.SS
   *-- Parameters..: cTime1   = Time string in format 
   *--                                    HH:MM:SS/HH:MM:SS.SS
   *--               xTime2   = Time string in format 
   *--                                    HH:MM:SS/HH:MM:SS.SS,
   *--                          or numeric value, a number of hours
   *--               nHours   = Numeric (number of hours to add to 
   *--                          cTime1
   *-------------------------------------------------------------------

      if type("xTime2") $ "FN"
         RETURN CLASS::Sec2Time( CLASS::Time2Sec(cTime1)+xTime2*3600 )
      endif

   RETURN CLASS::Sec2Time( CLASS::Time2Sec(cTime1)+;
                         CLASS::Time2Sec(xTime2) )
   *-- EoF: AddTimes()

   FUNCTION Civ2Mil( cCivilTime )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 11/09/1996
   *-- Notes.......: Converts string like "12:59 a.m." to standard 
   *--               24-hour HH:MM:SS. If the string contains neither 
   *--               "a" nor "p", the hours will not be converted.
   *-- Written for.: dBASE IV
   *-- Rev. History: 03/01/1992 -- Original 
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             VERSIONS. Jay Parsons, 72662,1302
   *--               11/09/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: time.Civ2Mil(<cCivilTime>)
   *-- Example.....: ? _app.Time.Civ2Mil("2:03:24 a.m.")
   *-- Returns.....: Character String
   *-- Parameters..: cCivilTime = Time to convert to 24 hour time.
   *-------------------------------------------------------------------
      
      local cTstring, nTime
      
      cTstring =  trim( lower( cCiviltime ) )
      nTime = val( cTstring )
      do case
         case "p" $ cTstring
            cTstring = left( cTstring, at( "p", cTstring ) - 1 )
            nTime = mod( nTime, 12 ) + 12
         case "a" $ cTstring
            cTstring = left( cTstring, at( "a", cTstring ) - 1 )
            nTime = mod( nTime, 12 )
      endcase
      cTstring = transform( nTime, "@L 99" ) ;
                 + trim( substr( cTstring, at( ":", cTstring )))
      
   RETURN cTstring + iif( len( cTstring ) = 5, ":00", "" )
   *-- EoF: Civ2Mil()

   FUNCTION DiffTime( cTime1, cTime2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 11/09/1996
   *-- Notes.......: Calculate difference between two times given as
   *--               HH:MM:SS or HH:MM:SS.SS strings.  If second time 
   *--               is smaller than first, assumes midnight passage. 
   *--               Returns HH:MM:SS string, or HH:MM:SS.SS string if 
   *--               fractional seconds passed.
   *-- Written for.: dBASE IV
   *-- Rev. History: 03/01/1992 -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/09/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: time.Time2Sec()
   *--               time.Sec2Time()
   *-- Called by...: Any
   *-- Usage.......: time.DiffTime(<cTime1>,<cTime2>)
   *-- Example.....: ? _app.Time.DiffTime("2:03:24","5:12:33")
   *-- Returns.....: Character String
   *-- Parameters..: cTime1 = Time to subtract from cTime2
   *--               cTime2 = Time to subtract from (larger value, 
   *--                        unless after midnite)
   *-------------------------------------------------------------------

   RETURN CLASS::Sec2Time( 86400 + CLASS::Time2Sec( cTime2 ) - ;
                                 CLASS::Time2Sec( cTime1 ) )
   *-- EoF: DiffTime()

   FUNCTION IsAmPm( cTime )
   *-------------------------------------------------------------------
   *-- Programmer..: Charles Miedzinksi
   *-- Date........: 11/09/1996
   *-- Notes.......: Taken from TechNotes (?), Checks to see if a 
   *--               character string is in proper AM/PM time format.
   *-- Written for.: dBASE IV 1.1, 1.5
   *-- Rev. History: 11/05/1992 -- Modified a bit for the Library ...
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/09/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Called by...: Any
   *-- Usage.......: time.IsAmPm(<cTime>)
   *-- Example.....: ? _app.Time.IsAmPm("20:15:05") && should return .F.
   *-- Returns.....: Logical
   *-- Parameters..: cTime = a time string in format: "HH:MM:SS a/pm"
   *--                   Seconds part of string is optional, as is A/PM
   *-------------------------------------------------------------------

   RETURN iif(val(left(cTime, 2)) >= 1 .and. ;
          val(left(cTime, 2)) <= 12, ;
          iif(val(substr(cTime, 4, 2)) >= 0 ;
          .and. val(substr(cTime, 4, 2)) <= 59, ;
          iif(substr(cTime, 6, 1) $ 'aApP', .T., ;
          iif(val(substr(cTimer, 6, 2)) >= 0 ;
          .and. val(substr(cTime, 6, 2)) <= 59, ;
          .T., .F.)), .F.), .F.)
   *-- EoF: IsAmPm()

   FUNCTION Mil2Civ( cMilTime )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 11/09/1996
   *-- Notes.......: Converts HH:MM:SS[.SS] 24-hour string to a.m./p.m.
   *--               notation
   *-- Written for.: dBASE IV
   *-- Rev. History: 03/01/1992 -- Original 
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/09/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: time.Mil2Civ(<cMilTime>)
   *-- Example.....: ? _app.Time.Mil2Civ("14:03:24")
   *-- Returns.....: Character String
   *-- Parameters..: cMilTime = 24-hour time to convert
   *-------------------------------------------------------------------
      
      local cCivtime, nHours, cMins
       
      cCivtime = ltrim( trim( cMiltime ) )
      nHours = val( cCivtime )
      cMins = substr( cCivtime, at( ":", cCivtime ) ) + " " ;
                 + iif( nHours > 11, "p.m.", "a.m." )
      
   RETURN ltrim( str( mod( nHours + 11, 12 ) + 1 ) ) + cMins
   *-- EoF: Mil2Civ()

   FUNCTION Sec2Time( nSeconds )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 11/09/1996
   *-- Notes.......: Convert number of seconds to time string in format
   *--               of HH:MM:SS or HH:MM:SS.SS.
   *-- Written for.: dBASE IV
   *-- Rev. History: 03/01/1992 -- Original 
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/09/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: time.Sec2Time(<nSeconds>)
   *-- Example.....: ? _app.Time.Sec2Time(30001.3)
   *-- Returns.....: Character String
   *-- Parameters..: nSeconds = Seconds to be converted 
   *-------------------------------------------------------------------
      
      local nHrs, nMins, nSecs, cTemp
       
      nSecs = mod( nSeconds, 86400 )
      nHrs  = int( nSecs / 3600 )
      nSecs = nSecs - nHrs * 3600
      nMins = int( nSecs / 60 )
      nSecs = nSecs - nMins * 60
      cTemp = transform( nHrs, "@L 99" ) + ":" ;
                 + transform( nMins, "@L 99" ) + ":"

   RETURN cTemp + iif( nSecs = int( nSecs ), ;
                       transform( nSecs, "@L 99" ), ;
                       transform( nSecs, "@L 99.99" ) )
   *-- EoF: Sec2Time()

   FUNCTION Time2Sec( cTime )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 11/09/1996
   *-- Notes.......: Convert HH:MM:SS or HH:MM:SS.SS string to seconds.
   *-- Written for.: dBASE IV
   *-- Rev. History: 03/01/1992 -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/09/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: time.Time2Sec(<cTime>)
   *-- Example.....: ? _app.Time.Time2Sec("01:24:15")
   *-- Returns.....: Numeric
   *-- Parameters..: cTime = Time string in format HH:MM:SS or 
   *--                      HH:MM:SS.SS
   *-------------------------------------------------------------------
      
      local cTemp, nSecs
      
      cTemp = cTime
      nSecs = 3600 * val( cTemp )
      cTemp = substr( cTemp, at( ":", cTemp ) + 1 )
      nSecs = nSecs + 60 * val( cTemp )
      
   RETURN nSecs + val( substr( cTemp, at( ":", cTemp ) + 1 ) )
   *-- EoF: Time2Sec()

ENDCLASS
